#include <CHM/CHMpreCompHead.h>
#pragma hdrstop

#include <CHM/CHMhl7ClientSocket.h>
#include <CHM/NETdll.h>
#include <CHM/CHMhl7ServerSocket.h>

struct CHMhl7ClientSocketPrivate
{
   CHMhl7ClientSocketPrivate():Handle(0){}
   NETconnectionHandle Handle;
   CHMboolean Dynamic;
};

CHMhl7ClientSocket::CHMhl7ClientSocket( CHMboolean Dynamic )
{
   CHMhl7ServerSocket::ensureInitialized();
   pMember = new CHMhl7ClientSocketPrivate;
   pMember->Dynamic = Dynamic;
   CHM_CHECK_CALL(NETtransportCreate( this, &pMember->Handle ));
}

CHMhl7ClientSocket::CHMhl7ClientSocket( int ClientId, CHMboolean Dynamic )
{
   CHMhl7ServerSocket::ensureInitialized();
   pMember = new CHMhl7ClientSocketPrivate;
   pMember->Handle = (NETconnectionHandle)ClientId;
   pMember->Dynamic = Dynamic;
}

CHMhl7ClientSocket::~CHMhl7ClientSocket()
{
   if( pMember->Handle )
   {
      CHM_CHECK_CALL(NETtransportCloseConnection( pMember->Handle ));
   }
   NETuserHandle UserHandle;
   CHM_CHECK_CALL(NETtransportDestroy( pMember->Handle, &UserHandle ));
   delete pMember;
}

/** Connect to a remote host. */
void CHMhl7ClientSocket::connect( const char* Host, short Port )
{
   CHM_CHECK_CALL(NETtransportConnect( pMember->Handle, Host, Port) );
}

/** Send a message. */
void CHMhl7ClientSocket::sendMessage( const char* Message )
{
   CHM_CHECK_CALL(NETtransportSendMessage( pMember->Handle, Message ));
}

/** Get the IP address */
CHMuint32 CHMhl7ClientSocket::getIpAddress()
{
   CHMuint32 Ip;
   CHM_CHECK_CALL(NETtransportGetIpAddress( pMember->Handle, &Ip ));
   return Ip;
}

CHMboolean CHMhl7ClientSocket::isConnected() const
{
   CHMuint32 Flag;
   CHM_CHECK_CALL(NETtransportIsConnected(pMember->Handle, &Flag));
   return Flag == 1;
}

/** Get the IP address as a string*/
const char* CHMhl7ClientSocket::getIpAddressString()
{
   const char* pIpAddress;
   CHM_CHECK_CALL(NETtransportGetIpAddressString( pMember->Handle, &pIpAddress ));
   return pIpAddress;
}

/** Gets the host name.*/
const char* CHMhl7ClientSocket::getHostName()
{
   const char* pHostName;
   CHM_CHECK_CALL(NETtransportGetHostName( pMember->Handle, &pHostName ));
   return pHostName;
}

/** Close a connection. */
void CHMhl7ClientSocket::closeConnection()
{
   CHM_CHECK_CALL(NETtransportCloseConnection( pMember->Handle ));
   if( pMember->Dynamic )
   {
      delete this;
   }
}

void CHMhl7ClientSocket::onClose( CHMhl7ClientSocket& Client )
{
   if( pMember->Dynamic )
   {
      delete this;
   }
}

int CHMhl7ClientSocket::id()
{
   return (int)pMember->Handle;
}
